# microdvd.pm v0.1
#
# Copyright (c) 2005 Joni Valtanen
# GNU General Public Licence v2
#
# Version information
#
# 0.01  -  051107, Joni Valtanen
#    * created

package Subtitle::Microdvd;

use 5.008004;
use strict;
use warnings;

require Exporter;
require Subtitle;
#require Subtitle::microdvd::file;
#require Subtitle::microdvd::edit;

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Subtitle ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = '0.01';


# Preloaded methods go here.

# Autoload methods go after =cut, and are processed by the autosplit program.
sub IsWritable
  {
   return 1;
}

sub IsFormat
  {
    my ($input) = @_;
    if( ( $input eq "MICRODVD" ) || ( $input eq "microdvd" ) )
      {
	return 1;
      }
    return 0;
  }

# tests file is microdvd format
#
# in: filename
# out: 1 - miccrodvd
#      0 - not microdvd
sub IsThis
  {
    my ($input) = @_;
    my $isMicrodvd=0;
    my @line_t1;
    my @line_t2;
    my @line_t3;
    my @line_t4;
    my $i;
    my $start=0;
    my $stop=0;
    my $text=0;
    my $line;

    open IFILE, "$input" or die("Couldn't open $input\n");

    for ($i=0; ( ($i<6) && ( $line = readline(IFILE)) ) ;$i++) {
      @line_t1 = split(/\n/,$line);
      if($line_t1[0])
      {
      @line_t1 = split(/\r/,$line_t1[0]);
      if($line_t1[0])
      {
	@line_t2 = split(/}/,$line_t1[0]);
	if($line_t2[2]) {
	@line_t3 = split(/{/,$line_t2[0]);
	@line_t4 = split(/{/,$line_t2[1]);

	if(&Subtitle::IsNumeric( $line_t3[1] ))
	{
	  $start = 1;
	}
	if(&Subtitle::IsNumeric( $line_t4[1] ))
	{
	  $stop = 1;
	}
	if( $line_t2[2] )
	{
	  $text = 1;
	}
      }
      }
      }
    }
    close IFILE;

    if($start && $stop && $text)
      {
	$isMicrodvd = 1;
      }
    return $isMicrodvd;
  }

sub ReadFile()
  {
    my ($input,$fps) = @_;

    my $line;
    my $i=0;
    my $test;
    my @line_t1;
    my @line_t2;
    my @line_t3;
    my @line_t4;
    my @sub;

    open IFILE, "$input" or die("Couldn't open $input\n") ;

    # reads whole file line by line
    # and writes it to stdout
    while ($line =  readline(IFILE)) {
      @line_t1 = split(/}/, $line);
      @line_t2 = split(/{/, $line_t1[0]);
      @line_t3 = split(/{/, $line_t1[1]);
      @line_t4 = split(/\r/,$line_t1[2]);
      @line_t4 = split(/\n/,$line_t4[0]);
      # drops negatives out ;)
      if ( $line_t2[1] > 0 ) {
	$sub[$i]{start} = $line_t2[1];
	$sub[$i]{end} = $line_t3[1];
	$sub[$i]{subtitle} = $line_t4[0];
	$sub[$i]{starttime} = &Subtitle::ConvertFPSToTime($line_t2[1], $fps );
	$sub[$i]{endtime} = &Subtitle::ConvertFPSToTime($line_t3[1], $fps );
	$i++;
      }

    }
      #close everything
      close IFILE;
    return ($i, @sub);
  }

# in: file to write, number of subtitles, @subtitles in mode $subtitle[number]{start|end|subtitle}
sub WriteFile()
  {
    my ($output, $lines, @subtitles) = @_;
    my $oldout;
    my $line;
    my $test;
    my $i=0;

    open $oldout, ">&STDOUT" or die "Can't duplicate STDOUT: $!";
    open STDOUT, ">$output" or die("Couldn't open $output\n") ;

    # writes whole file line by line
    while ($i<$lines) {
      $line = "{".int($subtitles[$i]{start})."}{".int($subtitles[$i]{end})."}".$subtitles[$i]{subtitle}."\r\n";
      print "$line";
      $i++;
    }

    #close everything
    close STDOUT;
    open STDOUT, ">&", $oldout or die "Can't restore old STDOUT: $!";
  }


sub WriteFileFromTo()
  {
    my ($output, $from, $to, $shift, @subtitles) = @_;

    my $i=0;
    my $j=0;
    my $line="";
    my $oldout;

  # Save subtitle file from frame to
  # from
  while ( $from > $subtitles[$i]{start} )
    {
      $i++;
    }

  $j=$i;

  while ( ( $to > $subtitles[$j]{start} ) )
	{
	  $j++;
	}

    open $oldout, ">&STDOUT" or die "Can't duplicate STDOUT: $!";
    open STDOUT, ">$output" or die("Couldn't open $output\n") ;

    # writes whole file line by line
#    for(; $i<$j; $i++){
    while ( ($i <= $j) ) {
      $line = "{".int( $subtitles[$i]{start} + $shift)."}{".int( $subtitles[$i]{end} + $shift )."}".$subtitles[$i]{subtitle}."\r\n";
      print "$line";
      $i++;
    }

    #close everything
    close STDOUT;
    open STDOUT, ">&", $oldout or die "Can't restore old STDOUT: $!";

  }
1;
__END__
# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Subtitle::Microdvd - Perl extension for identifying, loading and saving MicroDVD subtitles

=head1 SYNOPSIS

  use Subtitle::Microdvd;

  #load microdvd subtitle
   my $fps=25;
   my $filename="subtitle.sub";
   my ($lines, @sub) = &Subtitle::Microdvd::ReadFile($filename, $fps);
   for my $i (0...($lines-1))
   {
      print "start frame: @sub[$i]{start}\n";
      print "end frame: @sub[$i]{end}\n";
      print "start time: @sub[$i]{starttime}\n";
      print "end time: @sub[$i]{endtime}\n";
      print "subtitle: @sub[$i]{subtitle}\n";
   }

=head1 DESCRIPTION

Subtitle::Microdvd allows perl developer easily read and write Microdvd subtitles. Parsing is done by this extension.

=head2 EXPORT

None by default.



=head1 SEE ALSO

Perl(1)

Subtitle(3pm)
Subtitle::Microdvd(3pm)
Subtitle::Subviewer(3pm)

http://www.kapsi.fi/~jvaltane/subtitle/

=head1 AUTHOR

Joni Valtanen, E<lt>jvaltane@kapsi.fiE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005 by Joni Valtanen

This library is free software; you can redistribute it and/or modify
it under the GNU General Public Licence version 2.

Read COPYING from the package for further information.


=cut
